#
# WiKicker
#
#  MarkUpAsHtml.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: MarkUpAsHtml.pm,v 1.5 2004/01/03 12:40:55 naney Exp $
#
#
package WiKicker::App::MarkUpAsHtml;

use 5.005;
use strict;
use WiKicker::App::Configuration;
use WiKicker::WikiCGI::Environment;
use WiKicker::HTML;
use WiKicker::URI;
use WiKicker::Text::WordExtractor;
use WiKicker::LoadAverage;

sub new {
  my ($class, $properties_file, $url, $no_escape, $check_load_average) = @_;

  my $configuration = WiKicker::App::Configuration->new(properties_file =>
                                                        $properties_file);
  my $environment = WiKicker::WikiCGI::Environment->new(configuration => $configuration);
  my $self = bless { configuration => $configuration }, $class;
  $self->{no_escape} = $no_escape;

  if (defined $check_load_average && $check_load_average) {
    unless ($configuration->automatic_link_enabled(WiKicker::LoadAverage->new)) {
      $self->{no_automatic_link} = 1;
      return $self;
    }
    else {
      $self->{no_automatic_link} = 0;
    }
  }

  my $word_extractor = WiKicker::Text::WordExtractor->new;
  $environment->db->shared_lock_pages;
  my @page_list = $environment->db->page_list;
  $environment->db->unlock_pages;
  $word_extractor->set_words(\@page_list, 1);

  $self->{word_extractor} = $word_extractor;
  $self->{wikicker_url} = $url;

  return $self;
};

sub markup {
  my ($self, $text) = @_;

  return '' unless $text;

  if ($self->{no_automatic_link}) {
    return $self->_escape($text);
  }

  my $result = '';

  my $splited = $self->{word_extractor}->split_into_words($text);
  for my $item (@$splited) {
    if ($item->[0] == 0) {
      $result .= $self->_escape($item->[1]);
    }
    else {
      my $page_name = $item->[1];
      $page_name = $item->[2] if defined $item->[2];
      my $href = $self->_make_href($page_name);
      my $link_text = $self->_escape($item->[1]);
      $result .= qq(<a class="auto" href="$href">$link_text</a>);
    }
  }
  return $result;
}

sub _escape {
  my ($self, $text) = @_;

  if ($self->{no_escape}) {
    return $text;
  }
  else {
    return WiKicker::HTML::escape($text);
  }
}

sub _make_href {
  my ($self, $page_name) = @_;

  return WiKicker::URI::make_read_page_url($self->{wikicker_url}, $page_name);
}

1;
