#
# WiKicker
#
#  Response.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Response.pm,v 1.5 2004/06/20 05:56:43 naney Exp $
#
#

package WiKicker::CGI::Response;

use 5.005;
use strict;
#use CGI;

sub new {
  my ($class, %options) = @_;

  $| = 1;
  my $self = bless { memory => '',
                     enable_memory => 0,
                   }, $class;

  return $self;
}

sub start_memory {
  my $self = shift;

  $self->{memory} = '';
  $self->{enable_memory} = 1;
}

sub stop_memory {
  my $self = shift;

  $self->{enable_memory} = 0;
}

sub get_memory {
  my $self = shift;

  return $self->{memory};
}

sub http_header {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{http_header} = $value;
  }
  return $self->{http_header};
}

sub add_content {
  my ($self, $value) = @_;

  $self->{content} .= $value;

  if ($self->{enable_memory}) {
    $self->{memory} .= $value;
  }
}

sub content {
  my ($self, $value) = @_;
  if (defined $value) {
    $self->{content} = $value;
    if ($self->{enable_memory}) {
      $self->{memory} .= $value;
    }
  }
  return $self->{content};
}

sub content_length {
  length(shift->{content});
}

sub to_string {
  my $self = shift;

  $self->http_header . $self->content;
}

sub add_cookie {
  my ($self, @cookies) = @_;

  push(@{$self->{cookies}}, @cookies);
}

sub cookies {
  shift->{cookies};
}

sub flush {
  my $self = shift;

  if ($self->http_header) {
    print $self->http_header;
    $self->http_header('');
  }
  if ($self->content) {
    print $self->content;
    $self->content('');
  }
}

1;
