#
# WiKicker
#
#  Makefile.PL
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Configuration.pm,v 1.16 2004/08/03 12:55:51 naney Exp $
#
#

package WiKicker::Configuration;

use 5.005;
use strict;
use Carp;
use WiKicker::WRI::Factory;

sub new {
  my $class = shift;

  my $self = bless {}, $class;

  return $self;
};

sub properties {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{properties} = $value;
  }
  return $self->{properties};
}

sub wikicker_home {
  croak 'You must override WiKicker::Configuration::wikicker_home';
}

sub automatic_link_enabled {
  my ($self, $load_average) = @_;

  croak 'no load average object' unless defined $load_average;

  my $setting = $self->properties->get('automaticlink') || '';
  unless ($setting) {
    return 1;
  }

  if ($setting =~ /^\s*enable\s*$/) {
    return 1;
  }

  if ($setting =~ /^\s*load\s*<\s*(\d(?:\.\d+)?)\s*$/) {
    my $load = $1;
    if ($load <= 0) {
      return 0;
    }
    my $load_average_1 = $load_average->load_average_1;
    if ($load_average_1 < 0) {
      return 0;
    }
    if ($load_average_1 < $load) {
      return 1;
    }
    return 0;
  }

  return 0;
}

sub user_scheme_to_class_name {
  my ($self, $scheme) = @_;

 return $self->properties->get("RWRN.$scheme");
}

sub last_modified {
  my $self = shift;

  return $self->properties->last_modified;
}

sub stylesheet_uri_list {
  my ($self, $uri) = @_;

  eval "use URI";
  my $stylesheet = $self->properties->get('html.stylesheet');

  unless ($stylesheet) {
    return ();
  }

  my @css_list = split(/\s+/, $stylesheet);

  my @result = map {URI->new_abs($_, $uri) } @css_list;
  return @result;
}


1;
