#
# WiKicker
#
#  DiaryArticleHtmlFragmentVisitor.pm
#
#
# Copyright (C) 2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: DiaryArticleHtmlFragmentVisitor.pm,v 1.3 2004/12/13 12:53:46 naney Exp $
#
#

package WiKicker::Diki::DiaryArticleHtmlFragmentVisitor;

use strict;
use WiKicker::Diki::ArticleHtmlFragmentVisitor;

use vars qw(@ISA);
@ISA = qw(WiKicker::Diki::ArticleHtmlFragmentVisitor);

# override
sub _heading_level_adjust { 2 }

# override
sub heading_prefix_mark {
  my ($self, $level) = @_;

  if ($level != 1) {
    return '@';
  }

  my $id = $self->env->{article}->id;
  $id =~ /^(\d\d\d\d-\d\d)-(\d\d)/;
  my $ym = $1;
  my $d = $2;
  my $ten_day_range;
  if ($d <= 10) {
    $ten_day_range = "$ym-a";
  }
  elsif ($d <= 20) {
    $ten_day_range = "$ym-b";
  }
  else {
    $ten_day_range = "$ym-c";
  }
  my $ten_day_url = $self->env->environment->url . "/d/$ten_day_range.html#$id";
  my $article_url = $self->env->environment->url . "/d/$id.html";
  qq(<a name="$id" href="$article_url">■</a><a href="$ten_day_url">■</a>);
}

sub _end_article_body {
  my $self = shift;

  my $cgi
    = $self->env->configuration->properties->get('article.kuttukibbs.cgi') || '';
  my $js
    = $self->env->configuration->properties->get('article.kuttukibbs.js') || '';

  if ($cgi ne '' && $js ne '') {
    my $id = $self->env->{article}->id;

    $cgi =~ s/\$id/$id/g;
    $js  =~ s/\$id/$id/g;

    $self->{content} .= <<__ENDSTR__;
<div class="comment">
<div class="caption">[<a href="$cgi">コメントする / コメントを全部見る</a>]</div>
<!-- div class="commentshort" --><script language="JavaScript" type="text/javascript" src="$js"></script><!-- /div -->
</div>
__ENDSTR__
  }
}

# override
sub _visit_HeadingNode_hook {
  my ($self, $heading_node) = @_;

  if ($heading_node->level == 1) {
    my $id = $self->env->{article}->id;
    $id =~ /^(\d\d\d\d-\d\d-\d\d)/;

    $self->{title} = "$1 / ";

    eval "use WiKicker::WikiPage::GetSubjectVisitor";
    my $visitor = WiKicker::WikiPage::GetSubjectVisitor->new(wri_factory => $self->env->wri_factory);
    $visitor->visit($heading_node);
    $self->{title} .= $visitor->subject(40);
  }
}

1;
