#
# WiKicker
#
#  DiaryDay.pm
#
#
# Copyright (C) 2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: DiaryDay.pm,v 1.3 2004/12/13 12:53:46 naney Exp $
#
#

package WiKicker::Diki::DiaryDay;

use 5.005;
use strict;
use Carp;

sub new {
  my ($class, %param) = @_;

  croak 'no date' unless exists $param{date};
  croak "$class: invalid date: " . $param{date}
    unless $param{date} =~ /^(\d\d\d\d-\d\d-\d\d)$/;

  my $self = bless { date => $param{date} }, $class;

  return $self;
}

sub date { shift->{date} }
sub size { scalar(keys %{shift->{article}}) }

sub get_article {
  my ($self, $article_id) = @_;

  $article_id =~ /^\d\d\d\d-\d\d-\d\d-(.+)$/;
  return $self->{article}->{$1};
}

sub add_article {
  my ($self, $article) = @_;

  $self->{article}->{$article->short_id} = $article;
}

sub remove_article {
  my ($self, $article_id) = @_;

  unless ($article_id =~ /^(\d\d\d\d-\d\d-\d\d)-(.*)$/) {
    croak "Invalid article_id " . $article_id;
  }
  my $short_id = $2;

  unless (exists $self->{article}->{$short_id}) {
    Log::Log4perl->get_logger->error("no article $article_id");
    return;
  }
  delete $self->{article}->{$short_id};
}

sub top_article {
  my ($self, $article) = @_;

  my @list = keys %{$self->{article}};
  if (@list) {
    return $self->{article}->{$list[0]};
  }
  return undef;
}

# return {id => last_modified} hash reference
sub article_last_modified_set {
  my $self = shift;

  return $self->_get_article_property_map(sub { shift->last_modified });
}

# return {id => term_list_modified} hash reference

sub article_term_list_modified_set {
  my $self = shift;

  return
    $self->_get_article_property_map(sub { shift->term_list_modified });
}

sub _get_article_property_map {
  my ($self, $closure) = @_;

  my %result;
  for my $short_id (keys %{$self->{article}}) {
    my $article = $self->{article}->{$short_id};
    $result{$article->id} = &{$closure}($article);
  }
  return \%result;
}

sub to_html {
  my ($self, $url) = @_;

  my $date = $self->date;
  if ($self->size == 0) {
    return qq(<!-- no article day '$date' -->\n);
  }

  my ($y, $m, $d) = $date =~ /(\d\d\d\d)-(\d\d)-(\d\d)/;
  $m =~ s/^0//;
  $d =~ s/^0//;

  my $result;
  if (defined $url) {
  $result = <<__ENDSTR__;
<div class="day">
  <h2><a href="$url/d/$date.html">$y年$m月$d日</a></h2>
  <div class="body">
__ENDSTR__
}
  else {
  $result = <<__ENDSTR__;
<div class="day">
  <h2>$y年$m月$d日</h2>
  <div class="body">
__ENDSTR__
  }

  $result .= $self->_articles_to_html;
  $result .= <<__ENDSTR__;
  </div>
</div>
__ENDSTR__

  return $result;
}

sub _articles_to_html {
  my $self = shift;

  my $result = '';
  for my $short_id (sort { $self->{article}->{$b}->last_modified <=>
                             $self->{article}->{$a}->last_modified }
                    keys %{$self->{article}}) {
    $result .= $self->{article}->{$short_id}->to_html;
  }
  return $result;
}

sub article_list {
  my $self = shift;

  map {$self->{article}->{$_}} keys %{$self->{article}};
}

1;
