#
# WiKicker
#
#  KeywordArticle.pm
#
#
# Copyright (C) 2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: KeywordArticle.pm,v 1.2 2004/08/03 12:55:52 naney Exp $
#
#

package WiKicker::Diki::KeywordArticle;

use 5.005;
use strict;
use Carp;
use WiKicker::Diki::Article;
use WiKicker::Time;

use vars qw(@ISA);
BEGIN {
  @ISA = qw(WiKicker::Diki::Article);
}

sub _make_html {
  my ($self, $text, $term_list, $environment) = @_;

  my @lines = split("\n", $text);
  my $line;
  my @def = ();
  while (defined($line = shift(@lines))) {
    if ($line =~ /^\*/) {
      unshift(@lines, $line);
      last;
    }
    if ($line =~ /^-\s*def\s*:\s*(.*)$/) {
      my $term = $1;
      $term =~ s/\s+$//;
      push(@def, $term);
    }
  }
  if (@def) {
    $self->set_defined_term_list(\@def);
  }
  else {
    $self->set_defined_term_list([ $self->id ]);
  }
  $text = join("\n", @lines);

  my $document_node = $self->_make_document_node($text, $environment);
  return $self->_make_html_fragment($document_node, $environment, $term_list);
}

sub _make_html_fragment {
  my ($self, $document_node, $environment, $term_list) = @_;

  my $last_modified = WiKicker::Time::japanese_date_html_str($self->last_modified,
                                                             'JST');
  my $visitor =
    $self->_create_html_fragment_visitor($environment, $term_list);
  $visitor->visit($document_node);
  $self->{title} = $visitor->title;
  $self->set_explicit_term_list($visitor->env->explicit_term_list);
  $self->set_implicit_term_list($visitor->env->implicit_term_list);
  return $visitor->content;
}


sub _create_html_fragment_visitor {
  my ($self, $environment, $term_list) = @_;

  eval "use WiKicker::Diki::KeywordArticleHtmlFragmentVisitor";
  croak $@ if $@;
  my $visitor = WiKicker::Diki::KeywordArticleHtmlFragmentVisitor->new;
  $visitor->env($self->_create_html_env($environment, $term_list));
  return $visitor;
}

sub title {
  my $self = shift;

  return $self->{title};
}

sub date {
  my $self = shift;

  $self->id =~ /^(\d\d\d\d-\d\d-\d\d)/;
  return $1;
}

sub short_id {

  my $self = shift;

  $self->id =~ /^\d\d\d\d-\d\d-\d\d-(.*)$/;
  return $1;
}


sub _make_document_node {
  my ($self, $text, $environment) = @_;

  return $environment->create_document_parser->parse($text);
}

sub _check_serialize_field {
  my $self = shift;
}

sub _id_format_check {
  my $self = shift;

  croak 'no id' unless $self->{id};
}

sub set_defined_term_list {
  my ($self, $list) = @_;
  $self->{no_serialize_defined_term_list} = $list;
}

sub defined_term_list { shift->{no_serialize_defined_term_list} || [] }

1;
