#
# WiKicker
#
#  PrimaryKeywordDatabase.pm
#
#
# Copyright (C) 2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: PrimaryKeywordDatabase.pm,v 1.2 2004/08/03 12:55:52 naney Exp $
#
#

package WiKicker::Diki::PrimaryKeywordDatabase;

use 5.005;
use strict;
use Carp;

sub new {
  my ($class, %param) = @_;

  unless (exists $param{directory}) {
    croak "No directory";
  }

  my $self = bless { directory => $param{directory} }, $class;
}

sub directory { shift->{directory} }

sub keyword_id_modified_iterator {
  my $self = shift;

  return WiKicker::Diki::PrimaryKeywordDatabase::Iterator
    ->new(directory => $self->directory);

}

sub _get_file_name {
  my ($self, $keyword_id) = @_;

  return $self->directory . "/$keyword_id.dkk";
}

sub get_keyword_article {
    my ($self, $keyword_id) = @_;

  my $file_name = $self->_get_file_name($keyword_id);
  open(FILE, $file_name) || croak "Can't open file $file_name: $!";
  local $/;
  my $value = <FILE>;
  close(FILE);
  return $value;
}

sub exists_keyword {
  my ($self, $keyword_id) = @_;

  return -e $self->_get_file_name($keyword_id);
}

sub keyword_last_modified {
  my ($self, $keyword_id) = @_;

  my @stat = stat($self->_get_file_name($keyword_id));
  return $stat[9];
}

sub check_keywords {
  my ($self, $keyword_list) = @_;

  my @result = ();
  for my $article (@{$keyword_list->list}) {
    unless ($self->exists_keyword($article->id)) {
      push(@result, [ $article->id, 'deleted' ]);
    }
    my $modified = $self->keyword_last_modified($article->id);
    if ($modified > $article->last_modified) {
      push(@result, [ $article->id,
                      'updated',
                      $self->get_keyword_article($article->id),
                      $modified ]);
    }
  }

  if (@result) {
    return \@result;
  }
  else {
    return undef;
  }
}

package WiKicker::Diki::PrimaryKeywordDatabase::Iterator;

use 5.005;
use strict;
use Carp;
use Symbol;

sub new {
  my ($class, %param) = @_;

  unless (exists $param{directory}) {
    croak 'No directory';
  }

  my $handle = gensym;
  opendir($handle, $param{directory}) || croak "Can't open directory: $!";

  my $self = bless { directory => $param{directory},
                     handle => $handle }, $class;
  $self->_read_next;

  return $self;
}

sub DESTROY {
  my $self = shift;
  if (defined $self->{keyword_id}) {
    closedir($self->{handle});
  }
}

sub _read_next {
  my $self = shift;

  my $file_name;
  while (defined ($file_name = readdir($self->{handle}))) {
    next unless $file_name =~ /^(.+)\.dkk$/;
    $self->{keyword_id} = $1;
    my @stat = stat($self->{directory} . '/' . $file_name);
    $self->{last_modified} = $stat[9];
    return;
  }
  closedir($self->{handle});
  $self->{keyword_id} = undef;
  $self->{last_modified} = undef;
  return;
}

sub has_next {
  my $self = shift;

  return defined $self->{keyword_id};
}

sub get_next {
  my $self = shift;

  my @result = ($self->{keyword_id}, $self->{last_modified});
  if (defined $self->{keyword_id}) {
    $self->_read_next;
  }
  return @result;
}


1;
