#
# WiKicker
#
#  PrimaryYearDatabase.pm
#
#
# Copyright (C) 2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: PrimaryYearDatabase.pm,v 1.3 2004/12/13 12:53:46 naney Exp $
#
#

package WiKicker::Diki::PrimaryYearDatabase;

use 5.005;
use strict;
use Carp;
use WiKicker::Diki::PrimaryMonthDatabase;

sub new {
  my ($class, %param) = @_;

  croak "$class->new: no directory param" unless exists $param{directory};
  croak 'no year' unless exists $param{year};

  my $self = bless { directory => $param{directory},
                     year => $param{year} }, $class;
}

sub directory { shift->{directory} };

sub month_list {
  my $self = shift;

  if ($self->{month_list}) {
    return $self->{month_list};
  }

  opendir(DIR, $self->directory) || croak "Can't open directory: $!";
  my @files;
  my $file;
  while (defined ($file = readdir(DIR))) {
    if ($file =~ /^\d\d$/) {
      push(@files, $file);
    }
  }
  closedir(DIR);

  $self->{month_list} = [ sort @files ];
}

sub recent_month {
  my $self = shift;

  my $month_list = $self->month_list;
  if (scalar(@$month_list) == 0) {
    return '';
  }
  else {
    return $month_list->[-1];
  }
}

sub diary_month_directory {
  my ($self, $month) = @_;

  return $self->directory . '/' . $month;
}


sub month_db {
  my ($self, $month) = @_;

  if (exists $self->{month_db}->{$month}) {
    return $self->{month_db}->{$month};
  }

  return $self->{month_db}->{$month}
    = WiKicker::Diki::PrimaryMonthDatabase
      ->new(directory => $self->diary_month_directory($month),
            month => $month);
}


sub recent_diary_article_set {
  my ($self, $num_of_days) = @_;

  my $rest_days = $num_of_days;
  my %result = ();

  for my $month (reverse @{$self->month_list}) {
    my $month_db = $self->month_db($month);
    my ($day_count, $set) = $month_db->recent_diary_article_set($rest_days);
    $rest_days -= $day_count;
    for my $article_id (keys %$set) {
      $result{$article_id} = $set->{$article_id};
    }
    last if $rest_days <= 0;
  }

  return ($num_of_days - $rest_days, \%result);
}

sub article_diary_article_set {
  my ($self, $range) = @_;

  $range =~ /^\d\d\d\d-(\d\d)/;
  return $self->month_db($1)->article_diary_article_set($range);
}

sub one_day_diary_article_set {
  my ($self, $range) = @_;

  $range =~ /^\d\d\d\d-(\d\d)/;
  return $self->month_db($1)->one_day_diary_article_set($range);
}

sub ten_day_diary_article_set {
  my ($self, $range) = @_;

  $range =~ /^\d\d\d\d-(\d\d)/;
  return $self->month_db($1)->ten_day_diary_article_set($range);
}

sub get_article {
  my ($self, $article_id) = @_;

  $article_id =~ /^\d\d\d\d-(\d\d)/;
  return $self->month_db($1)->get_article($article_id);
}

1;
