#
# WiKicker
#
#  Environment.pm
#
#
# Copyright (C) 2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Environment.pm,v 1.2 2004/06/19 06:31:39 naney Exp $
#
#
package WiKicker::Environment;

use 5.005;
use strict;
use Carp;
use AutoLoader 'AUTOLOAD';

sub new {
  my ($class, %option) = @_;

  croak 'no configuration' unless exists $option{configuration};

  my $self = bless {
                    configuration => $option{configuration},
                   }, $class;

  return $self;
}

sub configuration { shift->{configuration} }

# Move from WiKicker::Configuration

sub create_document_parser {
  my $self = shift;

  eval "use WiKicker::WikiPage::DocumentParser";
  if ($@) {
    croak "Can't load WiKicker::WikiPage::DocumentParser: $@";
  }
  return WiKicker::WikiPage::DocumentParser->new(wri_factory => $self->create_wri_factory);
}

# Move from WiKicker::Configuration

sub create_wri_factory {
  my $self = shift;

  return WiKicker::WRI::Factory->new(user_rwrn_map_object => $self->configuration);
}

sub touch;

1;

__END__

sub touch {
  my $self = shift;

  my $touch_file = $self->configuration->properties->get('touch.file');
  return unless ($touch_file);

  unless(open(OUT, ">$touch_file")) {
    warn "Can't open touch file: $touch_file: $!";
    return;
  }
  print OUT time;
  close(OUT);
}
