#
# WiKicker
#
#  HTML.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: HTML.pm,v 1.4 2004/04/02 12:04:38 naney Exp $
#
#

package WiKicker::HTML;

use 5.005;
use strict;
use Carp;

use vars qw($_VERBOSE);

BEGIN {
  $_VERBOSE = 1;
}

=head1 NAME

WiKicker::HTML - HTML Utilities.

=head1 SYNOPSYS

  use WiKicker::HTML;

  my $escaped_text   = WiKicker::HTML::escape('a < b');
  my $attribute_text = WiKicker::HTML::escape_attribute('"dog"');

=head1 DESCRIPTION

HTML Utilities module.

=head1 SUBROUTINE

=over

=cut

=item escape(text)

escape as PCDATA.

=cut

sub escape {
  my $string = shift;

  unless (defined $string) {
    carp 'WiKicker::HTML::escape: undefined parameter' if $_VERBOSE;
    return $string;
  }

  $string =~ s/&/&amp;/g;
  $string =~ s/</&lt;/g;
  $string =~ s/>/&gt;/g;

  return $string;
}

=item escape_attribute(text)

escape as element attribute text.

=cut
sub escape_attribute {
  my $string = shift;

  unless (defined $string) {
    carp 'WiKicker::HTML::escape_attribute: undefined parameter' if $_VERBOSE;
    return $string;
  }

  #$string = escape($string);
  $string =~ s/&/&amp;/g;
  $string =~ s/</&lt;/g;
  $string =~ s/>/&gt;/g;
  # copy from escape() for speed up.

  $string =~ s/"/&quot;/g;

  return $string;
}

1;

__END__
=back

=head1 AUTHOR

WATANABE Yoshimasa E<lt>naney@naney.orgE<gt>

=cut
