#
# WiKicker
#
#  Lock.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Lock.pm,v 1.6 2004/08/03 12:41:21 naney Exp $
#
#

package WiKicker::Lock;

use 5.005;
use strict;
use FileHandle;
use Fcntl ':flock';

BEGIN {
  if ($^O eq 'solaris') {
    eval 'use File::Flock';
  }
  if ($@) {
    die q(Can't load File::Flock);
  }
}

sub parent_locker {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{parent_locker} = $value;
  }
  return $self->{parent_locker};
}

sub exclusive_lock_key {
  my ($self, $key) = @_;

  if (exists $self->{lock}->{$key}) {
    $self->_shift_lock_key($key, LOCK_EX);
  }
  else {
    $self->_new_lock_key($key, LOCK_EX);
  }
}

sub shared_lock_key {
  my ($self, $key) = @_;

  if (exists $self->{lock}->{$key}) {
    $self->_shift_lock_key($key, LOCK_SH);
  }
  else {
    $self->_new_lock_key($key, LOCK_SH);
  }
}

sub lock_mode_key {
  my ($self, $key) = @_;
  if (exists $self->{lock_mode}->{$key}) {
    return $self->{lock_mode}->{$key};
  }
  else {
    return 0;
  }
}

sub unlock_key {
  my ($self, $key) = @_;

  if ($^O eq 'solaris') {
    $self->_unlock_key_solaris($key);
    return;
  }
  if (exists $self->{lock}->{$key}) {
    flock($self->{lock}->{$key}, LOCK_UN);
    $self->{lock}->{$key}->close;
    delete $self->{lock}->{$key};
    delete $self->{lock_mode}->{$key};
    unlink $self->lock_file_name($key);
  }
}

sub _unlock_key_solaris {
  my ($self, $key) = @_;

  if (exists $self->{lock}->{$key}) {
    File::Flock::unlock($self->{lock}->{$key});
    delete $self->{lock}->{$key};
    delete $self->{lock_mode}->{$key};
  }
}

sub _new_lock_key {
  my ($self, $key, $operation) = @_;

  if ($^O eq 'solaris') {
    $self->_new_lock_key_solaris($key, $operation);
    return;
  }

  my $lock_file_name = $self->lock_file_name($key);
  #warn "lock $key - $lock_file_name";
  my $lock = new FileHandle;
  $lock->open(">$lock_file_name") || die "Can't open lockfile '$lock_file_name': $key: $!";
  for (1..120) {
    if (flock($lock, $operation | LOCK_NB)) {
      $self->{lock_mode}->{$key} = $operation;
      $self->{lock}->{$key} = $lock;
      return;
    }
    warn("can't lock: $!");
    sleep(1);
  }
  close($lock);
  unlink $lock_file_name;
  die q(Can't lock: Time out);
}

sub _new_lock_key_solaris {
  my ($self, $key, $operation) = @_;

  my $lock_file_name = $self->lock_file_name($key);
  for (1..120) {
    if (File::Flock::lock($lock_file_name, $operation == LOCK_SH, 'nonblocking')) {
      $self->{lock_mode}->{$key} = $operation;
      $self->{lock}->{$key} = $lock_file_name;
      return;
    }
    sleep(1);
  }
  die q(Can't lock: Time out);
}

sub _shift_lock_key {
  my ($self, $key, $operation) = @_;

  if ($^O eq 'solaris') {
    my $lock_file_name = $self->{lock}->{$key};
    for (1..120) {
      if (File::Flock::lock($lock_file_name, $operation == LOCK_SH, 'nonblocking')) {
        $self->{lock_mode}->{$key} = $operation;
        $self->{lock}->{$key} = $lock_file_name;
        return;
      }
      sleep(1);
    }
    die q(Can't lock: Time out);
  }


  my $lock = $self->{lock}->{$key};
  for (1..120) {
    if (flock($lock, $operation | LOCK_NB)) {
      $self->{lock_mode}->{$key} = $operation;
      $self->{lock}->{$key} = $lock;
      return;
    }
    sleep(1);
  }
  close($lock);
  unlink $self->lock_file_name($key);
  die q(Can't shift lock: Time out);
}


1;
