#
# WiKicker
#
#  PageNameList.pm
#
#
# Copyright (C) 2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: PageNameList.pm,v 1.4 2004/06/10 14:02:54 naney Exp $
#
#

package WiKicker::PageNameList;

use 5.005;
use strict;

sub new {
  my ($class, $page_name_list_ref) = @_;

  my $self = bless { page_name_list => [] }, $class;
  $self->{page_name_list} = $page_name_list_ref
    if defined $page_name_list_ref;
  return $self;
}

sub list {
  shift->{page_name_list};
}

sub has {
  my ($self, $name) = @_;

  unless (exists $self->{hash}) {
    my %hash;
    for my $page_name (@{$self->list}) {
      $hash{$page_name}++;
    }
    $self->{hash} = \%hash;
  }

  exists $self->{hash}->{$name};
}

sub ancestors_page_name_list {
  my ($self, $name) = @_;

  my @path = split('/', $name);
  my @result = ();
  my @path_name;
  for (my $i = 0; $i < @path - 1; $i++) {
    push(@path_name, $path[$i]);
    if ($self->has(join('/', @path_name))) {
      push(@result, join('/', @path_name));
    }
  }

  return @result;
}

sub page_list_with_predicate {
  my ($self, $predicate) = @_;

  return grep { &$predicate($_) } @{$self->list};
}

sub full_page_name_list {
  my ($self, $name) = @_;

  if (exists $self->{_full_page_name_list_cache}->{$name}) {
    return @{$self->{_full_page_name_list_cache}->{$name}};
  }

  my @full_page_name_list = $self->_full_page_name_list($name);
  $self->{_full_page_name_list_cache}->{$name} = \@full_page_name_list;

  return @full_page_name_list;
}

sub _full_page_name_list {
  my ($self, $name) = @_;

  my $page_name_list = $self->{page_name_list};
  my @result = ();
  $self->_last_segment_match(\@result, $name, $page_name_list);

  if (@result) {
    return @result;
  }

  my @segment = split('/', $name);
  if (@segment < 2) {
    return ();
  }

  my $re = join('(/.+)?/', map {"\Q$_\E"} @segment);
  $re = '^(.+/)?' . $re;
  $re .= '$';

  foreach my $target_name (@$page_name_list) {
    if ($target_name =~ m|$re|) {
      push(@result, $target_name);
    }
  }
  return @result;
}

sub _last_segment_match {
  my ($self, $result, $name, $page_name_list) = @_;

#  my $target_name;
#   foreach $target_name (@$page_name_list) {
#     if ($target_name =~ m|/\Q$name\E$|) {
#       push(@result, $target_name);
#     }
#   }
#---
#   my $substring = '/' . $name;
#   my $pattern_length = length($substring);
#   foreach my $target_name (@$page_name_list) {
#     my $pos = rindex($target_name, $substring);
#     if ($pos != -1 && (length($target_name) - $pattern_length) == $pos) {
#       push(@$result, $target_name);
#     }
#   }
  my $substring = '/' . $name;
  my $pos = length($substring) * -1;
  foreach my $target_name (@$page_name_list) {
    if (substr($target_name, $pos) eq $substring) {
      push(@$result, $target_name);
    }
  }
}

1;
