#
# WiKicker
#
#  PageNameTrie.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: PageNameTrie.pm,v 1.13 2004/08/03 12:55:52 naney Exp $
#
#


package WiKicker::Tree::PageNameTrie;

use strict;
use WiKicker::UTF8;

sub new {
  my $class = shift;

  my $self = bless [0], $class;

  return $self;
}

sub add {
  my ($self, $page_name) = @_;

  # It must be recognized by InlineParser.
  if ($page_name =~ m|^[A-Z][a-z]+(?:[A-Z][a-z]+)+(?:(?:/[A-Z][a-z]+(?:[A-Z][a-z]+)+)+)*$|o) {
    return;
  }

  #my @c_list = WiKicker::UTF8::split_into_array($page_name);
  #
  #if (@c_list >= 2) {
  #  $self->_add(\@c_list, \$page_name);
  #}
  my $c_list_ref = WiKicker::UTF8::split_into_array_ref($page_name);

  if (@$c_list_ref >= 2) {
    $self->_add($c_list_ref, \$page_name);
  }
}

sub _add {
  my ($self, $characters_ref, $page_name_ref) = @_;

  my $transition = 1;
  while (1) {
    unless (@$characters_ref) {
      $self->[$transition - 1] = $$page_name_ref; # accept
      return;
    }
    my $c = pop @$characters_ref;

    if ($transition == @$self) {
      push(@$characters_ref, $c);
      $self->_add_extend($characters_ref, $page_name_ref, $transition);
      return;
    }

    unless (ref $self->[$transition]) {
      if ($self->[$transition] eq $c) {
        $transition += 2;
        next;
      }
      else {
        # split
        my $current_c = $self->[$transition];
        my @current_list = @{$self}[($transition + 1)..(@$self - 1)];
        my $current_list_ref = \@current_list;
        bless $current_list_ref, ref $self;
        $self->[$transition] = { $current_c => $current_list_ref };
        splice(@$self, $transition + 1);

        my $new_node = WiKicker::Tree::PageNameTrie->new;
        $self->[$transition]->{$c} = $new_node;
        $new_node->_add($characters_ref, $page_name_ref);
        return;
      }
    }
    else {
      # hash
      unless (exists $self->[$transition]->{$c}) {
        $self->[$transition]->{$c} = WiKicker::Tree::PageNameTrie->new;
      }
      $self->[$transition]->{$c}->_add($characters_ref, $page_name_ref);
      return;
    }
  }
}

sub _add_extend {
  my ($self, $characters_ref, $page_name_ref, $transition) = @_;

  $self->[$transition + ((@$characters_ref) * 2 - 1)] = undef; # extend
  for (my $i = @$characters_ref - 1; $i >= 0; $i--) {
    $self->[$transition] = $characters_ref->[$i];
    $transition++;
    $self->[$transition] = undef;
    $transition++;
  }
  $self->[$transition - 1] = $$page_name_ref; #accept
}


sub lookup {
  my ($array_ref, $first, $next, $trie) = @_;

  return _lookup($array_ref, $first, $next, $trie, 0);
}

sub _lookup {
  my ($array_ref, $first, $next, $trie, $number) = @_;

  my $transition = 1;
  my $result = [];
  while (1) {
    if ($trie->[$transition - 1]) {
      if ($number >= 2) {
        unshift(@$result, [$number, $trie->[$transition - 1]]);
      }
    }

    if ($next < $first) {
      return $result;
    }

    my $c = $array_ref->[$next];
    $next--;
    $number++;

    if ($transition == @$trie) {
      return $result;
    }
    unless (ref $trie->[$transition]) {
      if ($trie->[$transition] eq $c) {
        $transition += 2;
        next;
      }
      else {
        return $result;
      }
    }
    else {
      # hash
      if (exists $trie->[$transition]->{$c}) {
        my $child_result = _lookup($array_ref, $first, $next,
                                   $trie->[$transition]->{$c}, $number);
        unshift(@$result, @$child_result);
      }
      return $result;
    }
  }
}

sub mark_abbrev {
  my $trie = shift;

  _mark_abbrev($trie, 1);
}

sub _mark_abbrev {
  my ($trie, $transition) = @_;

  if (@$trie == $transition) {
    return $trie->[$transition - 1]; # leaf, return page name
  }

  unless (ref $trie->[$transition]) {
    my $next_transition = $transition;
    my $full_name;
    while (1) {
      $next_transition += 2;
      if (@$trie == $next_transition) {
        $full_name = $trie->[$next_transition - 1]; # leaf
        $next_transition -= 2;
        last;
      }
      elsif (ref $trie->[$next_transition]) {
        $full_name = _mark_abbrev($trie, $next_transition);
        $next_transition -= 2;
        last;
      }
    }
    return undef unless defined $full_name;
    while ($next_transition >= $transition) {
      if ($trie->[$next_transition] eq '/'
          && !($trie->[$next_transition - 1])) {
        $trie->[$next_transition - 1] = $full_name;
      }
      $next_transition -= 2;
    }
    return $full_name;
  }
  else {
    # hash
    my $name_count = 0;
    my $name = '';
    for my $c (keys %{$trie->[$transition]}) {
      my $full_name = _mark_abbrev($trie->[$transition]->{$c}, 1);
      if (defined $full_name) {
        if ($c eq '/' && !($trie->[$transition - 1])) {
          $trie->[$transition - 1] = $full_name;
        }
        $name_count++;
        $name = $full_name;
      }
    }

    if ($name_count == 1) {
      return $name;
    }
    return undef;
  }
}

1;
