#
# WiKicker
#
#  URI.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: URI.pm,v 1.7 2004/08/11 15:01:45 naney Exp $
#
#

package WiKicker::URI;

use strict;
use URI;

sub escape_URI {
  my $uri = shift;

  $uri =~ s/(\W)/'%' . unpack('H2', $1)/eg;
  return $uri;
}

sub unescape_URI {
  my $uri = shift;

  $uri =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack('H2', $1)/eg;
  return $uri;
}

sub encode_x_www_form_urlencoded {
  my $uri = shift;

  $uri =~ s/([^\w ])/'%' . unpack('H2', $1)/eg;
  $uri =~ tr/ /+/;
  return $uri;
}

sub decode_x_www_form_urlencoded {
  my $uri = shift;

  $uri =~ tr/+/ /;
  $uri =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack('H2', $1)/eg;
  return $uri;
}

# RFC2396 3.3
sub page_name_to_path_segments {
  my $page_name = shift;

  $page_name =~ s|([^A-Za-z0-9-_.!~*'():@&=+\$./])|'%' . unpack('H2', $1)|eg;
  return $page_name;
}

sub make_read_page_url {
  my ($url, $page_name) = @_;

  my $suffix = '.html';

  # @todo move this if block to appropriate position.
  if ($page_name eq 'RssPage') {
    $suffix = '';
  }

  return $url . '/' . page_name_to_path_segments($page_name) . $suffix;
}

# sub make_track_back_url {
#   my ($url, $page_name) = @_;

#   return $url . '/' . page_name_to_path_segments($page_name) . '.TrackBack';
# }

sub short_http_url {
  my ($url, $max) = @_;

  my $original_length = length($url);
  if ($original_length <= $max) {
    return $url;
  }

  my $uri = URI->new($url);
  my $scheme = $uri->scheme;
  my $authority = $uri->authority;
  my @path = $uri->path_segments;
  my $query = $uri->query || '';
  my $fragment = $uri->fragment || '';

  return $url if is_host_only_uri($uri);

  if (length($fragment) > 3) {
    $uri->fragment('...');
    $fragment = '...';
    my $result = $uri->as_string;
    if (length($result) <= $max) {
      return $result;
    }
  }
  if ((length($query) + ($fragment ? length($fragment) + 1 : 0)) > 3) {
    if ($fragment) {
      $uri->fragment(undef);
    }
    $uri->query('...');
    $query = '...';
    my $result = $uri->as_string;
    if (length($result) <= $max) {
      return $result;
    }
  }

  if ($uri->query || $uri->fragment || (@path >= 2 && $path[-1] eq 'index.html')) {
    $uri->query(undef);
    $uri->fragment(undef);
    while (@path > 2) {
      $path[-1] = '...';
      $uri->path_segments(@path);
      my $result = $uri->as_string;
      if (length($result) <= $max) {
        return $result;
      }
      pop(@path);
    }
    if (@path == 2) {
      $path[1] = '...';
      $uri->path_segments(@path);
      my $result = $uri->as_string;
      if (length($result) <= $max) {
        return $result;
      }
    }
    $path[0] = '';
    $path[1] = '...';
    $uri->path_segments(@path);
    return $uri->as_string;
  }
  else {
    return _short_http_url_remove_from_parent($uri, $max);
  }
}

sub is_host_only_uri {
  my $uri = shift;

  my @path = $uri->path_segments;

  return (@path == 0 || (@path == 2 && $path[1] eq ''))
    && !($uri->query) && !($uri->fragment);
}

sub _short_http_url_remove_from_parent {
  my ($uri, $max) = @_;

  $uri->query(undef);
  $uri->fragment(undef);

  my @path = $uri->path_segments;
  while (@path >  2) {
    $path[1] = '...';
    $uri->path_segments(@path);
    my $result = $uri->as_string;
    if (length($result) <= $max) {
      return $result;
    }
    splice(@path, 1, 1);
  }

  if (@path == 2) {
    $path[1] = '...';
    $uri->path_segments(@path);
    my $result = $uri->as_string;
    if (length($result) <= $max) {
      return $result;
    }
  }

  $path[0] = '';
  $path[1] = '...';
  $uri->path_segments(@path);
  return $uri->as_string;
}

1;
