#
# WiKicker
#
#  UTF8.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: UTF8.pm,v 1.6 2004/08/03 12:45:39 naney Exp $
#
#

package WiKicker::UTF8;

use strict;
use Carp;
use AutoLoader 'AUTOLOAD';

use vars qw(@ISA);
use vars qw($parent_class $singleton);

BEGIN {
  if ($] >= 5.008) {
    $parent_class = 'WiKicker::UTF8_5_8';
  }
  else {
    $parent_class = 'WiKicker::UTF8_Unicode_String';
  }

  my $file = $parent_class;
  $file =~ s|::|/|g;
  $file .= '.pm';

  eval "use $parent_class" unless $INC{$file};
  die $@ if $@;

  unshift @ISA, $parent_class;
}

sub new {
  my $class = shift;

  return bless {}, $class;
}

sub substr_utf8 {
  my ($expr, $offset, $length) = @_;

  return $singleton->_substr_utf8($expr, $offset, $length);
}

sub length_utf8 {
  my $string = shift;

  return $singleton->_length_utf8($string);
}

#sub split_into_array {
#  my $string = shift;
#
#  return $singleton->_split_into_array($string);
#}

sub split_into_array {
  my $text = shift;

  my @result;
  while ($text =~ /([\x00-\x7f]|[\x80-\xdf].|[\xe0-\xef]..|[\xf0-\xff]...)/og) {
    push(@result, $1);
  }
  return @result;
}

sub split_into_array_ref {
  my $text = shift;

  my @result;
  while ($text =~ /([\x00-\x7f]|[\x80-\xdf].|[\xe0-\xef]..|[\xf0-\xff]...)/og) {
    push(@result, $1);
  }
  return \@result;
}

sub last_char {
  my $text = shift;

  #return '' if !defined($text) || $text eq '';

  $text =~ /([\x00-\x7f]|[\xc0-\xdf][\x80-\xbf]|[\xe0-\xef][\x80-\xbf]{2}|[\xf0-\xf7][\x80-\xbf]{3})$/;
  return $1;
#   if ($1 ne '') {
#     return $1;
#   }

#   return $singleton->_substr_utf8($text, -1, 1);
}

$singleton = new WiKicker::UTF8;



use vars qw($JCODE_LOADED);
BEGIN {
  $JCODE_LOADED = 0;
}

sub encode;
sub decode;
sub decode_from_unknown;

1;

__END__

sub encode {
  my ($charset, $text) = @_;

  unless ($JCODE_LOADED) {
    eval "use Jcode";
    if ($@) {
      croak "Can't load Jcode: $@";
    }
    $JCODE_LOADED = 1;
  }

  Jcode::convert(\$text, $charset, 'utf8');
  return $text;
}

sub decode {
  my ($charset, $text) = @_;

  unless ($JCODE_LOADED) {
    eval "use Jcode";
    if ($@) {
      croak "Can't load Jcode: $@";
    }
    $JCODE_LOADED = 1;
  }

  Jcode::convert(\$text, 'utf8', $charset);
}

sub decode_from_unknown {
  my $text = shift;

  unless ($JCODE_LOADED) {
    eval "use Jcode";
    if ($@) {
      croak "Can't load Jcode: $@";
    }
    $JCODE_LOADED = 1;
  }

  Jcode::convert(\$text, 'utf8');
}
