#
# WiKicker
#
#  User.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: User.pm,v 1.5 2004/01/12 01:35:10 naney Exp $
#
#

package WiKicker::User;

use 5.005;
use strict;

sub DEFAULT_USER_TIMEZONE      { 'GMT' }
sub DEFAULT_USER_EDITOR_WIDTH  { 80 }
sub DEFAULT_USER_EDITOR_HEIGHT { 20 }

sub new {
  my $class = shift;

  my $self = bless {}, $class;

  $self->{timezone} = DEFAULT_USER_TIMEZONE;
  $self->{name} = '';
  $self->{editor_width} = DEFAULT_USER_EDITOR_WIDTH;
  $self->{editor_height} = DEFAULT_USER_EDITOR_HEIGHT;
  $self->{preferences_changed} = 0;

  return $self;
}

sub id {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{id} = $value;
  }
  return $self->{id};
}

sub name {
  my ($self, $value) = @_;

  if (defined $value) {
    $value = normalize_name($value);
    if ($self->{name} ne $value) {
      $self->touch;
    }
    $self->{name} = $value;
  }

  return $self->{name};
}


sub normalize_name {
  my $value = shift;

  $value =~ s/^\s+//;
  $value =~ s/\s+$//;
  $value =~ s/\s+/ /g;

  return $value;
}

sub editor_width {
  my ($self, $value) = @_;

  if (defined $value) {
    if ($value <= 10) {
      $value = 10;
    }
    if ($value >= 300) {
      $value = 300;
    }
    if ($self->{editor_width} != $value) {
      $self->touch;
    }
    $self->{editor_width} = $value;
  }

  return $self->{editor_width};
}

sub editor_height {
  my ($self, $value) = @_;

  if (defined $value) {
    if ($value <= 1) {
      $value = 1;
    }
    if ($value >= 200) {
      $value = 200;
    }
    if ($self->{editor_height} != $value) {
      $self->touch;
    }
    $self->{editor_height} = $value;
  }

  return $self->{editor_height};
}

sub timezone {
  my ($self, $value) = @_;

  if (defined $value) {
    if ($self->{timezone} ne $value) {
      $self->touch;
      $self->{timezone} = $value;
    }
  }
  return $self->{timezone};
}

sub touch {
  my $self = shift;

  $self->{preferences_changed} = time;
}

sub preferences_changed {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{preferences_changed} = $value;
  }

  return $self->{preferences_changed};
}

1;
