#
# WiKicker
#
#  WRI.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: WRI.pm,v 1.5 2004/04/02 12:11:49 naney Exp $
#
#

package WiKicker::WRI;

use strict;
use WiKicker::HTML;
use WiKicker::ISBN;

sub new {
  my ($class, $original) = @_;

  my $self = bless {}, $class;

  $self->original($original);

  $self->_initialize;

  return $self;
}

sub _initialize {
}

sub original {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{original} = $value;
  }
  $self->{original};
}

sub to_string {
  return shift->original;
}

sub canonical {
  return undef;
}

sub to_subject_string {
  my ($self, $env) = @_;

  if ($self->get_image_url($env)) {
    return '';
  }

  return $self->to_string;
}

sub to_html {
  my ($self, $content, $env) = @_;

  return WiKicker::HTML::escape($content || $self->to_string);
}

sub _make_url_link_html {
  my ($self, $content, $env) = @_;

  my $image_url = $self->get_image_url($env);
  if ($image_url) {
    return $self->_make_image_link_html($content, $image_url, $env);
  }
  else {
    my $url = WiKicker::HTML::escape_attribute($self->get_link_url($env));
    if ($url) {
      my $isbn = $self->is_isbn;
      my $content_string;
      if (!($content) && $isbn) {
        $content_string = WiKicker::HTML::escape($isbn);
      }
      else {
        $content_string = WiKicker::HTML::escape($content || $self->to_string);
      }
      return qq(<a href="$url">$content_string</a>);
    }
    else {
      return WiKicker::HTML::escape($content || $self->to_string);
    }
  }
}

sub _make_image_link_html {
  my ($self, $content, $image_url, $env) = @_;
  my $href = $self->get_link_url($env);
  my $alt_string = WiKicker::HTML::escape($content || $self->to_string);

  my $scheme = $self->scheme;
  my $ismap = '';
  if ($scheme eq 'imagemap') {
    $ismap = ' ismap';
  }

  if ($href) {
    return qq(<a href="$href"><img$ismap src="$image_url" alt="$alt_string"></a>);
  }
  else {
    return qq(<img src="$image_url" alt="$alt_string">);
  }
}

sub is_isbn {
  my $self = shift;

  my $canonical = $self->canonical;
  if (defined $canonical) {
    if ($canonical->isa('WiKicker::WRI::URN') && $canonical->nid eq 'isbn') {
      return 'ISBN' . WiKicker::ISBN::hyphenation($canonical->nss);
    }
  }
  else {
    if ($self->isa('WiKicker::WRI::URN') && $self->nid eq 'isbn') {
      return 'urn:isbn:' . WiKicker::ISBN::hyphenation($self->nss);
    }
  }
  return '';
}

sub get_link_url {
  my ($self, $env) = @_;

  return '';
}

sub get_image_url {
  my ($self, $env) = @_;

  return '';
}


sub _make_page_name_link {
  my ($self, $page_name, $fragment_identifier, $content, $mtime_flag, $subject_flag, $env) = @_;

#   $env->link_maker->make_page_name_link($page_name,
#                                         $fragment_identifier,
#                                         $content,
#                                         $mtime_flag, $subject_flag);
  my ($link, $linked) =
    $env->link_maker->make_page_name_link_and_linked($page_name,
                                                     $fragment_identifier,
                                                     $content,
                                                     $mtime_flag, $subject_flag);

  $env->add_explicit_term(@$linked);

  return $link;
}



1;
