#
# WiKicker
#
#  Environment.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Environment.pm,v 1.8 2004/08/03 12:55:52 naney Exp $
#
#
package WiKicker::WikiCGI::Environment;

use 5.005;
use strict;
use Carp;
use WiKicker::DB::File;
use WiKicker::WRI::Resolver;
use WiKicker::Environment;
use WiKicker::File;
use AutoLoader 'AUTOLOAD';

use vars qw(@ISA);
@ISA = qw(WiKicker::Environment);

sub DESTROY {
  my $self = shift;

  eval {
    if ($self->{_cache}) {
      $self->{_cache}->disconnect_all;
    }
  };
  if ($@) {
    warn $@;
  }
}

# Move from WiKicker::Configuration

sub db {
  my $self = shift;

  if (exists $self->{db}) {
    return $self->{db};
  }
  return $self->{db} =
    WiKicker::DB::File->new($self->configuration->database_directory);
}

# Move from WiKicker::Configuration

sub resolver {
  my $self = shift;

  if ($self->{resolver}) {
    return $self->{resolver};
  }
  else {
    $self->{resolver} = WiKicker::WRI::Resolver->new;
    $self->{resolver}->db($self->db);
    $self->{resolver}->amazon_id($self->configuration->properties->get('Amazon.co.jp.AssociatesID'));
    return $self->{resolver};
  }
}

sub last_modified_for_view {
  my $self = shift;

  my $db_last_modified = $self->db->db_last_modified_time;
  my $config_last_modified = $self->configuration->last_modified;

  if ($db_last_modified > $config_last_modified) {
    return $db_last_modified;
  }
  return $config_last_modified;
}

sub html_fragment_cache_key {
  my ($self, $name) = @_;

  #return $WiKicker::VERSION . ':h:' . $name;
  return $WiKicker::VERSION . ':h:' . WiKicker::File::key_to_file_name($name);
}

sub _cache;
sub get_cache_value;
sub set_cache_value;
sub add_recent_log;
sub get_recent_log;

1;

__END__

sub _cache {
  my $self = shift;

  if (exists $self->{enable_cache}) {
    if ($self->{enable_cache}) {
      return $self->{_cache};
    }
    else {
      return undef;
    }
  }

  my $type = $self->configuration->properties->get('memorycache.type');
  unless (defined $type && $type eq 'memcached') {
    $self->{enable_cache} = 0;
    return undef;
  }
  my $server_text = $self->configuration->properties->get('memcached.servers');
  $server_text =~ s/^\s+//;
  $server_text =~ s/\s+$//;
  my @servers = split(/\s+/, $server_text);
  if (@servers == 0) {
    $self->{enable_cache} = 0;
    return undef;
  }

  eval "use Cache::Memcached";
  if ($@) {
    warn $@;
    $self->{enable_cache} = 0;
    return undef;
  }

  $self->{_cache} = Cache::Memcached->new({'servers' => \@servers,
                                           'select_timeout' => 2.0});
  $self->{enable_cache} = 1;

  return $self->{_cache};
}

sub get_cache_value {
  my ($self, $key) = @_;

  my $cache = $self->_cache;
  return undef unless defined $cache;

  return $cache->get($key);
}

sub set_cache_value {
  my ($self, $key, $value, $expire) = @_;

  my $cache = $self->_cache;
  return unless defined $cache;

  return $cache->set($key, $value, $expire);
}

sub add_recent_log {
  my ($self, $request) = @_;

  my $type = $self->configuration->properties->get('recentlog.type');
  return unless defined($type) && $type eq 'shared';
  my $key = $self->configuration->properties->get('recentlog.shared.key');
  return unless $key;
  eval "use WiKicker::Log::SharedRecentLog";
  if ($@) {
    warn $@;
    return;
  }
  my $log = WiKicker::Log::SharedRecentLog->new(ipc_key => $key);
  $log->add_log($request->user->id,
                $request->user->name,
                $request->name);
}

sub get_recent_log {
  my $self = shift;

  my $type = $self->configuration->properties->get('recentlog.type');
  return unless $type eq 'shared';
  my $key = $self->configuration->properties->get('recentlog.shared.key');
  return unless $key;
  eval "use WiKicker::Log::SharedRecentLog";
  if ($@) {
    return;
  }
  my $log = WiKicker::Log::SharedRecentLog->new(ipc_key => $key);
  return $log->get_log;
}
