#
# WiKicker
#
#  Page.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Page.pm,v 1.12 2004/08/03 12:55:52 naney Exp $
#
#

package WiKicker::WikiCGI::Page;

use 5.005;
use strict;
use WiKicker;
use WiKicker::PageName;
use WiKicker::Time;
use WiKicker::CGI::Page;
use AutoLoader 'AUTOLOAD';
use WiKicker::File;

use vars qw(@ISA);
BEGIN {
  @ISA = qw(WiKicker::CGI::Page);
}


# short cut
sub name {
  shift->request->name;
}

# short cut
sub db {
  shift->request->environment->db;
}

sub last_modified_for_send {
  my $self = shift;
  my $http_last_modified = $self->request->environment->last_modified_for_view;
  my $user_http_last_modified = $self->request->user->preferences_changed;
  if ($http_last_modified < $user_http_last_modified) {
    return $user_http_last_modified;
  }
  return $http_last_modified;
}

sub set_http_last_modified_for_send {
  my $self = shift;

  $self->request->attribute('http_last_modified_for_send',
                            $self->last_modified_for_send);
}

sub is_modified {
  my $self = shift;

  my $if_modified_since = $self->request->query->http('If-Modified-Since');
  if (defined $if_modified_since && $if_modified_since) {
    my $date = WiKicker::Time::RFC1123_to_gmtime($if_modified_since);
    if ($date && $date >= $self->last_modified_for_send) {
      return 0;
    }
  }
  return 1;
}

sub _unknown_action_message {
  my $self = shift;

  q(Unknown action ') . $self->request->action . q(' for ) . $self->name;
}

sub do_search;
sub _search;
sub _search_cache_key;

1;

__END__
sub do_search {
  my $self = shift;

  my $search_text = $self->name;
  my $cache_key = $self->_search_cache_key($search_text);
  my $cached_data = $self->request->environment->get_cache_value($cache_key);

  my $busy = $self->request->configuration->is_busy_to_search($self->request->load_average);

  $self->db->shared_lock_pages;

  $self->request->attribute('no_cache_content', 1);

  my $last_modified = $self->request->environment->last_modified_for_view;
  if (defined $cached_data) {
    if ($cached_data->{t} >= $last_modified) {
      my @result = @{$cached_data->{d}};
      my $content
        = join('', map { '- ' . WiKicker::PageName::armor_name($_) . "\n" }
               sort @result);
      $self->request->attribute('content', $content);
      $self->request->attribute('head_comments', [ 'search cache hit! ' . $cached_data->{t} ]);
      return 'read';
    }
    else {
      if ($busy) {
        my @result = @{$cached_data->{d}};
        my $content
          = join('', map { '- ' . WiKicker::PageName::armor_name($_) . "\n" }
                 sort @result);
        $self->request->attribute('content', $content);
        $self->request->attribute('head_comments', [ 'use old cache. ' . $cached_data->{t} ]);
        return 'read';
      }
    }

  }

  if ($busy) {
    return 'busy';
  }

  my @result = $self->_search($search_text);
  $self->request->environment->set_cache_value($cache_key,
                                               { t => $last_modified,
                                                 d => \@result },
                                               60 * 60 * 24);
  my $content
    = join('', map { '- ' . WiKicker::PageName::armor_name($_) . "\n" }
           sort @result);
  $self->request->attribute('content', $content);

  return 'read';
}

sub _search_cache_key {
  my ($self, $search_text) = @_;

  #return $WiKicker::VERSION . ':s:' . $search_text;
  return $WiKicker::VERSION . ':s:' . WiKicker::File::key_to_file_name($search_text);
}

sub _search {
  my ($self, $name) = @_;

  my $case_sensitive = 0;
  if ($name =~ /[A-Z]/) {
    $case_sensitive = 1;
  }

  my $db = $self->db;

  my @result;
  if ($case_sensitive) {
    for my $target_page (@{$db->page_name_list->list}) {
      if (index($target_page, $name) >= 0) {
        push(@result, $target_page);
        next;
      }

      my $target_page_content = $db->page($target_page);
      if (index($target_page_content, $name) >= 0) {
        push(@result, $target_page);
      }
    }
  }
  else {
    my $pattern = qr/\Q$name\E/i;
    for my $target_page (@{$db->page_name_list->list}) {
      if ($target_page =~ /$pattern/) {
        push(@result, $target_page);
        next;
      }

      my $target_page_content = $db->page($target_page);
      if ($target_page_content =~ /$pattern/) {
        push(@result, $target_page);
      }
    }
  }
  return @result;
}
