#
# WiKicker
#
#  Edit.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Edit.pm,v 1.7 2004/08/03 12:55:52 naney Exp $
#
#

package WiKicker::WikiCGI::View::HTML::Edit;

use 5.005;
use strict;

use WiKicker::WikiCGI::View::HTML;

use vars qw(@ISA);
@ISA = qw(WiKicker::WikiCGI::View::HTML);

sub make_html_body {
  my $self = shift;

  $self->response
    ->add_content($self->make_edit_form($self->request->attribute('content')));
}

sub make_edit_form {
  my ($self, $wiki_page) = @_;

  my $query = $self->request->query;

  my $user = $self->request->user;

  my $row = $user->editor_height;
  my $column = $user->editor_width;

  my $last_modified
    = $self->db->last_modified($self->request->name);

  my $section = $self->request->attribute('section');
  unless (defined $section) {
    $section = '';
  }
  my $has_section = 0;
  if (length($section) > 0) {
    $has_section = 1;
  }

  my $result;
  $result .= "<!-- begin edit form -->\n";
  $result .= qq(<div style="width: 100%">); # for IE bug
  $result .= $query->start_form(-method => 'POST',
                                -action => $self->request->url);
  $result .= '  ';
  # hidden
  $result .= $query->hidden(-name => 'name',
                            -default => $self->request->name,
                            -override => 1);
  $result .= "\n  ";
  $result .= $query->hidden(-name => 'action',
                            -default => 'write',
                            -override => 1);
  $result .= "\n  ";
  $result .= $query->hidden(-name => 'lastmodified',
                            -default => $last_modified);
  $result .= "\n  ";
  if ($has_section) {
    $result .= $query->hidden(-name => 'section',
                              -default => $section,
                              -override => 1);
    $result .= "\n  ";
  }
  # top button
  $result .= $self->_make_button_bar($query);
  $result .= "\n  ";
  $result .= '<br>';
  $result .= "\n  ";
  # text area
  $result .= $query->textarea(-name => 'content',
                              -default => $wiki_page,
                              -override => 1,
                              -rows => $row,
                              -columns => $column,
                              -style => 'width: 100%',
                             );
  # bottom button
  $result .= "\n  ";
  $result .= '<br>';
  $result .= qq(Your name: );
  $result .= $query->textfield(-name => 'un',
                               -default => $self->request->user->name,
                               -override => 1,
                               -size => 20,
                               -maxlength => 120);
  $result .= '<br>';
  $result .= "\n  ";
  $result .= $self->_make_button_bar($query);
  $result .= $query->end_form;
  $result .= qq(</div>);
  $result .= "\n<!-- end edit form -->\n";

  return $result;
}

sub _make_button_bar {
  my ($self, $query) = @_;

  my $request = $self->request;
  my $preview_text = $request->maketext('preview');
  my $cancel_text = $request->maketext('cancel');
  my $save_text = $request->maketext('save');

  my $result = '';
  $result .= "<!-- begin button -->\n  ";
  $result .= $query->submit(-name => 'action.preview', -value => $preview_text);
  $result .= "\n  ";
  $result .= $query->submit(-name => 'action.cancel',  -value => $cancel_text);
  $result .= "\n  ";
  $result .= $query->submit(-name => 'action.commit',  -value => $save_text);
  $result .= "\n  <!-- end button -->";

  return $result;
}

1;
