#
# WiKicker
#
#  Read.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Read.pm,v 1.15 2004/06/06 03:58:44 naney Exp $
#
#

package WiKicker::WikiCGI::View::HTML::Read;

use 5.005;
use strict;
use WiKicker::WikiCGI::View::HTML;
use WiKicker::URI;
use WiKicker::WikiPage::HtmlTocVisitor;
use AutoLoader 'AUTOLOAD';


use vars qw(@ISA);
@ISA = qw(WiKicker::WikiCGI::View::HTML);

sub make_html_body {
  my $self = shift;

  my $request = $self->request;
  my $response = $self->response;

#   if ($request->attribute('normalpage')) {
#     $response->add_content($self->_make_trackback_rdf);
#   }

  my $set_cache = $request->is_default_view_user;
  if (($request->attribute('revision'))) {
    $set_cache = 0;
  }
  if ($request->attribute('no_cache_content')) {
    $set_cache = 0;
  }

  if ($set_cache) {
    $response->start_memory;
  }
  $self->_make_html_fragment_with_toc;
  if ($set_cache) {
    my $memory = $response->get_memory;
    my $expires = 60 * 60 * 120;
    unless ($request->configuration->automatic_link_enabled($request->load_average)) {
      $expires = 60 * 60 * 30;
    }

    $response->stop_memory;

    $request->environment
      ->set_cache_value($request->environment->html_fragment_cache_key($request->name),
                        { t => $request->environment->last_modified_for_view,
                          c => $memory },
                        60 * 60 * 30);
  }
}

# sub _make_trackback_rdf {
#   my $self = shift;

#   my $url = WiKicker::URI::make_read_page_url($self->request->query->url,
#                                               $self->request->name);
#   my $url2 = $self->request->query->url . '?' . WiKicker::URI::escape_URI($self->request->name);
#   my $ping_url = $self->track_back_ping_url;
#   my $title = WiKicker::HTML::escape_attribute($self->request->name);

#   return <<__ENDSTR__;
# <!--
# <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
#          xmlns:dc="http://purl.org/dc/elements/1.1/"
#          xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">
# <rdf:Description
#     rdf:about="$url"
#     dc:identifer="$url"
#     dc:title="$title"
#     trackback:ping="$ping_url" />
# </rdf:RDF>

# <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
#          xmlns:dc="http://purl.org/dc/elements/1.1/"
#          xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">
# <rdf:Description
#     rdf:about="$url2"
#     dc:identifer="$url2"
#     dc:title="$title"
#     trackback:ping="$ping_url" />
# </rdf:RDF>
# -->
# __ENDSTR__
# }


sub enable_header_toolbar_edit {
  my $self = shift;
  unless ($self->request->attribute('editable')) {
    return 0;
  }
  if ($self->request->attribute('revision')) {
    return 0;
  }

  return 1;
}

sub enable_header_toolbar_diff {
  my $self = shift;
  unless ($self->request->attribute('editable')) {
    return 0;
  }

  return 1;
}

sub enable_header_toolbar_copy {
  my $self = shift;
  unless ($self->request->attribute('editable')) {
    return 0;
  }

  if ($self->request->attribute('revision')) {
    return 0;
  }
  else {
    return 1;
  }
}

sub enable_header_toolbar_create {
  return 1;
}

sub enable_header_toolbar_raw {
  my $self = shift;
  return $self->request->attribute('editable')
    || $self->request->attribute('revision');
}

sub enable_header_toolbar_rlog {
  my $self = shift;
  return $self->request->attribute('editable')
    || $self->request->attribute('revision');
}

1;

sub _make_html_fragment_with_toc;
sub _make_toc_from_document_node;

__END__

sub _make_html_fragment_with_toc {
  my $self = shift;

  my $document_node = $self->
    _make_document_node($self->request->attribute('content'));
  unless ($document_node) {
    return q(<p>Sorry, page parse error.</p>);
  }
  $self->response
    ->add_content($self->_make_toc_from_document_node($document_node));
  $self->response->flush;

  if ($self->request->attribute('revision')) {
    $self->_add_and_flush_html_fragment($document_node, 1);
  }
  else {
    $self->_add_and_flush_html_fragment($document_node, 0);
  }
}

sub _make_toc_from_document_node {
  my ($self, $document_node) = @_;

  my $visitor = WiKicker::WikiPage::HtmlTocVisitor->new;
  $visitor->visit($document_node);
  if ($visitor->size <= 1) {
    return '';
  }

  return qq(<div class="toc">\n) . $visitor->content . qq(</div>\n);
}
