#
# WiKicker
#
#  Node.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Node.pm,v 1.5 2004/08/03 12:55:52 naney Exp $
#
#

package WiKicker::WikiPage::Node;

use WiKicker::WikiPage::Acceptor;
use vars qw(@ISA);
@ISA = qw(WiKicker::WikiPage::Acceptor);

sub new {
  my ($class, %options) = @_;

  my $self = bless { child_node => [] }, $class;
}

sub size {
  my $self = shift;

  return @{$self->{child_node}} + 0;
}

sub add {
  my ($self, @nodes) = @_;
  push(@{$self->{child_node}}, @nodes);
}

sub remove_last_node {
  my $self = shift;
  if (@{$self->{child_node}} == 0) {
    die q(Internal Error: Can't remove last node - no node);
  }
  pop(@{$self->{child_node}});
}

# @deprecated
sub nodes {
  return @{shift->{child_node}};
}

# firster than nodes();
sub child_nodes {
  return shift->{child_node};
}

sub insert_before {
  my ($self, $new_node, $reference_node) = @_;

  my @new_nodes;
  my $inserted;
  for my $node (@{$self->{child_node}}) {
    if ($node == $reference_node) {
      push(@new_nodes, $new_node, $node);
      $inserted++;
    }
    else {
      push(@new_nodes, $node);
    }
  }
  unless ($inserted) {
    push(@new_nodes, $new_node);
  }

  $self->{child_node} = [ @new_nodes ];
}

sub insert_after {
  my ($self, $new_node, $reference_node) = @_;

  my @new_nodes;
  my $inserted;
  for my $node (@{$self->{child_node}}) {
    if ($node == $reference_node) {
      push(@new_nodes, $node, $new_node);
      $inserted++;
    }
    else {
      push(@new_nodes, $node);
    }
  }
  unless ($inserted) {
    unshift(@new_nodes, $new_node);
  }

  $self->{child_node} = [ @new_nodes ];
}

1;
