#
# WiKicker
#
#  ReplaceSectionVisitor.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: ReplaceSectionVisitor.pm,v 1.4 2003/11/01 11:59:50 naney Exp $
#
#

package WiKicker::WikiPage::ReplaceSectionVisitor;

use strict;

use WiKicker::WikiPage::TraverseVisitor;
use vars qw(@ISA);
@ISA = qw(WiKicker::WikiPage::TraverseVisitor);

sub new {
  my $class = shift;

  my $self = bless { new_raw => "" }, $class;

  return $self;
}

sub section_number {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{section_number} = $value;
  }
  return $self->{section_number};
}

sub new_section {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{new_section} = $value;
  }
  return $self->{new_section};
}

sub visit_DocumentNode {
  my ($self, $document_node) = @_;

  my $number = 0;
  my $replaced = 0;
  for my $node ($document_node->nodes) {
    if ($node->isa('WiKicker::WikiPage::HeadingNode')) {
      if ($node->level <= 2) {
        $number++;
      }
    }
    if ($number == $self->section_number) {
      unless ($replaced) {
        $self->{new_raw} .= $self->new_section;
        if ($self->new_section !~ /\n$/) {
          $self->{new_raw} .= "\n";
        }
        $replaced = 1;
      }
    }
    else {
      $self->{new_raw} .= $node->to_string;
    }
  }
  unless ($replaced) {
    $self->{new_raw} .= $self->new_section;
    if ($self->new_section !~ /\n$/) {
      $self->{new_raw} .= "\n";
    }
  }
}

sub new_raw {
  my $self = shift;

  return $self->{new_raw};
}

1;
